@@ -4,13 +4,12 @@ import os
 from dataclasses import dataclass
 from typing import List
 
-
 # Messages --------------------------------------------------------------------
 
 MESSAGES_DOC_TEMPLATE = """
 # Full List of Messages
 
-xplr messages categorized based on their purpose.
+xplr [messages][1] categorized based on their purpose.
 
 ## Categories
 
@@ -20,7 +19,9 @@ xplr messages categorized based on their purpose.
 
 ## Also See:
 
-- [Message](message.md)
+- [Message][1]
+
+[1]: message.md
 """.strip()
 
 CONFIGURATION_DOC_TEMPLATE = """
@@ -134,6 +135,7 @@ def gen_configuration():
     - node_types.md
     - layouts.md
     - modes.md
+    - modes.md
     """
 
     path = "./src/init.lua"
@@ -157,6 +159,7 @@ def gen_configuration():
             line.startswith("-- # Configuration ")
             or line.startswith("-- ## Config ")
             or line.startswith("-- ## Function ")
+            or line.startswith("-- ## On Load ")
         ):
             reading = configuration
 
@@ -216,6 +219,48 @@ def gen_configuration():
         print(doc, file=f)
 
 
+# xplr.util -------------------------------------------------------------------
+
+
+@dataclass
+class Function:
+    doc: List[str]
+    name: str
+
+
+def gen_xplr_util():
+
+    path = "./src/lua/util.rs"
+
+    functions: List[Function] = []
+
+    with open(path) as f:
+        lines = iter(f.read().splitlines())
+
+    reading = None
+
+    for line in lines:
+        if line.startswith("///"):
+            if reading:
+                reading.doc.append(line[4:])
+            else:
+                reading = Function(doc=[line[4:]], name="")
+
+        if line.startswith("pub fn") and reading:
+            reading.name = "\n### xplr.util." + line.split("<")[0].split()[-1] + "\n"
+            functions.append(reading)
+            reading = None
+            continue
+
+    with open("./docs/en/src/xplr.util.md", "w") as f:
+        for function in functions:
+            print(function.name)
+            print(function.name, file=f)
+
+            print("\n".join(function.doc))
+            print("\n".join(function.doc), file=f)
+
+
 def format_docs():
     os.system("prettier --write docs/en/src")
 
@@ -223,6 +268,7 @@ def format_docs():
 def main():
     gen_messages()
     gen_configuration()
+    gen_xplr_util()
     format_docs()
 
 
