@@ -1,4 +1,5 @@
 use crate::config::Config;
+use crate::config::Hooks;
 use crate::config::Mode;
 pub use crate::directory_buffer::DirectoryBuffer;
 use crate::explorer;
@@ -8,6 +9,7 @@ pub use crate::msg::in_::external::Command;
 pub use crate::msg::in_::external::ExplorerConfig;
 pub use crate::msg::in_::external::NodeFilter;
 pub use crate::msg::in_::external::NodeFilterApplicable;
+use crate::msg::in_::external::NodeSearcher;
 pub use crate::msg::in_::external::NodeSorter;
 pub use crate::msg::in_::external::NodeSorterApplicable;
 pub use crate::msg::in_::ExternalMsg;
@@ -22,6 +24,7 @@ use anyhow::{bail, Result};
 use chrono::{DateTime, Local};
 use gethostname::gethostname;
 use indexmap::set::IndexSet;
+use path_absolutize::*;
 use serde::{Deserialize, Serialize};
 use std::collections::HashMap;
 use std::collections::VecDeque;
@@ -125,6 +128,8 @@ impl History {
 pub struct LuaContextHeavy {
     pub version: String,
     pub pwd: String,
+    pub initial_pwd: String,
+    pub vroot: Option<String>,
     pub focused_node: Option<Node>,
     pub directory_buffer: Option<DirectoryBuffer>,
     pub selection: IndexSet<Node>,
@@ -142,6 +147,8 @@ pub struct LuaContextHeavy {
 pub struct LuaContextLight {
     pub version: String,
     pub pwd: String,
+    pub initial_pwd: String,
+    pub vroot: Option<String>,
     pub focused_node: Option<Node>,
     pub selection: IndexSet<Node>,
     pub mode: Mode,
@@ -158,11 +165,16 @@ pub struct InputBuffer {
     pub prompt: String,
 }
 
-#[derive(Debug, Clone, Serialize, Deserialize)]
+#[derive(Debug, Serialize, Deserialize)]
 pub struct App {
+    pub bin: String,
     pub version: String,
     pub config: Config,
+    pub hooks: Hooks,
+    pub vroot: Option<String>,
+    pub initial_vroot: Option<String>,
     pub pwd: String,
+    pub initial_pwd: String,
     pub directory_buffer: Option<DirectoryBuffer>,
     pub last_focus: HashMap<String, Option<String>>,
     pub selection: IndexSet<Node>,
@@ -183,24 +195,27 @@ pub struct App {
 
 impl App {
     pub fn create(
+        bin: String,
+        vroot: Option<PathBuf>,
         pwd: PathBuf,
         lua: &mlua::Lua,
         config_file: Option<PathBuf>,
         extra_config_files: Vec<PathBuf>,
     ) -> Result<Self> {
-        let mut config = lua::init(lua)?;
+        let (mut config, hooks) = lua::init(lua)?;
+        let mut hooks = hooks.unwrap_or_default();
 
         let config_file = if let Some(path) = config_file {
             Some(path)
         } else if let Some(dir) = dirs::home_dir() {
-            let path = dir.join(".config").join("xplr").join("init.lua");
+            let path = dir.join(".config/xplr/init.lua");
             if path.exists() {
                 Some(path)
             } else {
                 None
             }
         } else {
-            let path = PathBuf::from("/").join("etc").join("xplr").join("init.lua");
+            let path = PathBuf::from("/etc/xplr/init.lua");
             if path.exists() {
                 Some(path)
             } else {
@@ -215,8 +230,11 @@ impl App {
         let mut load_errs = vec![];
         for config_file in config_files {
             match lua::extend(lua, &config_file.to_string_lossy()) {
-                Ok(c) => {
+                Ok((c, maybe_hooks)) => {
                     config = c;
+                    if let Some(h) = maybe_hooks {
+                        hooks = hooks.extend(h);
+                    }
                 }
                 Err(e) => {
                     load_errs.push(e.to_string());
@@ -285,18 +303,37 @@ impl App {
         };
 
         let hostname = gethostname().to_string_lossy().to_string();
+
+        if let Some(vroot) = vroot.as_ref() {
+            if !pwd.starts_with(vroot) {
+                bail!(
+                    "{:?} is outside of virtual root {:?}",
+                    pwd.to_string_lossy(),
+                    vroot.to_string_lossy()
+                )
+            }
+        }
+
         let pwd = pwd.to_string_lossy().to_string();
+        let vroot = vroot.map(|v| v.to_string_lossy().to_string());
+        let initial_vroot = vroot.clone();
         env::set_current_dir(&pwd)?;
 
+        let initial_pwd = pwd.clone();
+
         let input = InputBuffer {
             buffer: Default::default(),
             prompt: config.general.prompt.format.clone().unwrap_or_default(),
         };
 
         let mut app = Self {
+            bin,
             version: VERSION.to_string(),
             config,
+            vroot,
+            initial_vroot,
             pwd,
+            initial_pwd,
             directory_buffer: Default::default(),
             last_focus: Default::default(),
             selection: Default::default(),
@@ -313,6 +350,7 @@ impl App {
             history: Default::default(),
             last_modes: Default::default(),
             hostname,
+            hooks,
         };
 
         let has_errs = !load_errs.is_empty();
@@ -406,12 +444,20 @@ impl App {
                 FocusByIndex(i) => self.focus_by_index(i),
                 FocusByIndexFromInput => self.focus_by_index_from_input(),
                 FocusByFileName(n) => self.focus_by_file_name(&n, true),
+                ScrollUp => self.scroll_up(),
+                ScrollDown => self.scroll_down(),
+                ScrollUpHalf => self.scroll_up_half(),
+                ScrollDownHalf => self.scroll_down_half(),
                 ChangeDirectory(dir) => self.change_directory(&dir, true),
                 Enter => self.enter(),
                 Back => self.back(),
                 LastVisitedPath => self.last_visited_path(),
                 NextVisitedPath => self.next_visited_path(),
                 FollowSymlink => self.follow_symlink(),
+                SetVroot(p) => self.set_vroot(&p),
+                UnsetVroot => self.unset_vroot(),
+                ToggleVroot => self.toggle_vroot(),
+                ResetVroot => self.reset_vroot(),
                 SetInputPrompt(p) => self.set_input_prompt(p),
                 UpdateInputBuffer(op) => self.update_input_buffer(op),
                 UpdateInputBufferFromKey => self.update_input_buffer_from_key(key),
@@ -441,9 +487,13 @@ impl App {
                 SwitchLayoutBuiltin(mode) => self.switch_layout_builtin(&mode),
                 SwitchLayoutCustom(mode) => self.switch_layout_custom(&mode),
                 Call(cmd) => self.call(cmd),
+                Call0(cmd) => self.call0(cmd),
                 CallSilently(cmd) => self.call_silently(cmd),
+                CallSilently0(cmd) => self.call_silently0(cmd),
                 BashExec(cmd) => self.bash_exec(cmd),
+                BashExec0(cmd) => self.bash_exec0(cmd),
                 BashExecSilently(cmd) => self.bash_exec_silently(cmd),
+                BashExecSilently0(cmd) => self.bash_exec_silently0(cmd),
                 CallLua(func) => self.call_lua(func),
                 CallLuaSilently(func) => self.call_lua_silently(func),
                 LuaEval(code) => self.lua_eval(code),
@@ -474,6 +524,10 @@ impl App {
                 ReverseNodeSorters => self.reverse_node_sorters(),
                 ResetNodeSorters => self.reset_node_sorters(),
                 ClearNodeSorters => self.clear_node_sorters(),
+                SearchFuzzy(p) => self.search_fuzzy(p),
+                SearchFuzzyFromInput => self.search_fuzzy_from_input(),
+                AcceptSearch => self.accept_search(),
+                CancelSearch => self.cancel_search(),
                 EnableMouse => self.enable_mouse(),
                 DisableMouse => self.disable_mouse(),
                 ToggleMouse => self.toggle_mouse(),
@@ -594,7 +648,7 @@ impl App {
             dir.focus = 0;
 
             if save_history {
-                if let Some(n) = self.clone().focused_node() {
+                if let Some(n) = self.focused_node() {
                     self.history = history.push(n.absolute_path.clone())
                 }
             }
@@ -635,7 +689,7 @@ impl App {
         Ok(self)
     }
 
-    fn focus_previous_by_relative_index(mut self, index: usize) -> Result<Self> {
+    pub fn focus_previous_by_relative_index(mut self, index: usize) -> Result<Self> {
         let mut history = self.history.clone();
         if let Some(dir) = self.directory_buffer_mut() {
             if let Some(n) = dir.focused_node() {
@@ -680,7 +734,7 @@ impl App {
         Ok(self)
     }
 
-    fn focus_next_by_relative_index(mut self, index: usize) -> Result<Self> {
+    pub fn focus_next_by_relative_index(mut self, index: usize) -> Result<Self> {
         let mut history = self.history.clone();
         if let Some(dir) = self.directory_buffer_mut() {
             if let Some(n) = dir.focused_node() {
@@ -719,10 +773,62 @@ impl App {
         }
     }
 
+    fn set_vroot(mut self, path: &String) -> Result<Self> {
+        let vroot = PathBuf::from(path).absolutize()?.to_path_buf();
+
+        if vroot.is_dir() {
+            self.vroot = Some(vroot.to_string_lossy().to_string());
+            if !PathBuf::from(&self.pwd).starts_with(&vroot) {
+                self.change_directory(path, true)
+            } else {
+                Ok(self)
+            }
+        } else {
+            self.log_error(format!(
+                "not a valid directory: {}",
+                vroot.to_string_lossy()
+            ))
+        }
+    }
+
+    fn unset_vroot(mut self) -> Result<Self> {
+        self.vroot = None;
+        Ok(self)
+    }
+
+    fn toggle_vroot(self) -> Result<Self> {
+        if self.vroot.is_some() && self.vroot == self.initial_vroot {
+            self.unset_vroot()
+        } else if self.vroot.is_some() && self.initial_vroot.is_some() {
+            self.reset_vroot()
+        } else if self.vroot.is_some() {
+            self.unset_vroot()
+        } else {
+            let vroot = self.pwd.clone();
+            self.set_vroot(&vroot)
+        }
+    }
+
+    fn reset_vroot(mut self) -> Result<Self> {
+        if let Some(vroot) = self.initial_vroot.clone() {
+            self.set_vroot(&vroot)
+        } else {
+            self.vroot = None;
+            Ok(self)
+        }
+    }
+
     fn change_directory(mut self, dir: &str, save_history: bool) -> Result<Self> {
-        let mut dir = PathBuf::from(dir);
-        if dir.is_relative() {
-            dir = PathBuf::from(self.pwd.clone()).join(dir);
+        let dir = PathBuf::from(dir).absolutize()?.to_path_buf();
+
+        if let Some(vroot) = &self.vroot.clone() {
+            if !dir.starts_with(vroot) {
+                return self.log_error(format!(
+                    "{:?} is outside of virtual root {:?}",
+                    dir.to_string_lossy(),
+                    vroot,
+                ));
+            }
         }
 
         match env::set_current_dir(&dir) {
@@ -731,6 +837,7 @@ impl App {
                 let focus = self.focused_node().map(|n| n.relative_path.clone());
                 self = self.add_last_focus(pwd, focus)?;
                 self.pwd = dir.to_string_lossy().to_string();
+                self.explorer_config.searcher = None;
                 if save_history {
                     self.history = self.history.push(format!("{}/", self.pwd));
                 }
@@ -741,29 +848,31 @@ impl App {
     }
 
     fn enter(self) -> Result<Self> {
-        self.focused_node()
-            .map(|n| n.absolute_path.clone())
-            .map(|p| self.clone().change_directory(&p, true))
-            .unwrap_or(Ok(self))
+        if let Some(path) = self.focused_node().map(|n| n.absolute_path.clone()) {
+            self.change_directory(&path, true)
+        } else {
+            Ok(self)
+        }
     }
 
     fn back(self) -> Result<Self> {
-        PathBuf::from(self.pwd.clone())
+        if let Some(p) = PathBuf::from(self.pwd.clone())
             .parent()
-            .map(|p| self.clone().change_directory(&p.to_string_lossy(), true))
-            .unwrap_or(Ok(self))
+            .and_then(|p| p.to_str())
+        {
+            self.change_directory(p, true)
+        } else {
+            Ok(self)
+        }
     }
 
     fn last_visited_path(mut self) -> Result<Self> {
         self.history = self.history.visit_last();
-        if let Some(target) = self.history.peek() {
-            if target.ends_with('/') {
-                target
-                    .strip_suffix('/')
-                    .map(|s| self.clone().change_directory(s, false))
-                    .unwrap_or(Ok(self))
+        if let Some(target) = self.history.peek().cloned() {
+            if let Some(path) = target.strip_suffix('/') {
+                self.change_directory(path, false)
             } else {
-                self.clone().focus_path(target, false)
+                self.focus_path(&target, false)
             }
         } else {
             Ok(self)
@@ -772,14 +881,11 @@ impl App {
 
     fn next_visited_path(mut self) -> Result<Self> {
         self.history = self.history.visit_next();
-        if let Some(target) = self.history.peek() {
-            if target.ends_with('/') {
-                target
-                    .strip_suffix('/')
-                    .map(|s| self.clone().change_directory(s, false))
-                    .unwrap_or(Ok(self))
+        if let Some(target) = self.history.peek().cloned() {
+            if let Some(path) = target.strip_suffix('/') {
+                self.change_directory(path, false)
             } else {
-                self.clone().focus_path(target, false)
+                self.focus_path(&target, false)
             }
         } else {
             Ok(self)
@@ -927,11 +1033,28 @@ impl App {
         }
     }
 
+    pub fn scroll_up(mut self) -> Result<Self> {
+        self.msg_out.push_back(MsgOut::ScrollUp);
+        Ok(self)
+    }
+
+    pub fn scroll_down(mut self) -> Result<Self> {
+        self.msg_out.push_back(MsgOut::ScrollDown);
+        Ok(self)
+    }
+
+    pub fn scroll_up_half(mut self) -> Result<Self> {
+        self.msg_out.push_back(MsgOut::ScrollUp);
+        Ok(self)
+    }
+
+    pub fn scroll_down_half(mut self) -> Result<Self> {
+        self.msg_out.push_back(MsgOut::ScrollDownHalf);
+        Ok(self)
+    }
+
     pub fn focus_path(self, path: &str, save_history: bool) -> Result<Self> {
-        let mut pathbuf = PathBuf::from(path);
-        if pathbuf.is_relative() {
-            pathbuf = PathBuf::from(self.pwd.clone()).join(pathbuf);
-        }
+        let pathbuf = PathBuf::from(path).absolutize()?.to_path_buf();
         if let Some(parent) = pathbuf.parent() {
             if let Some(filename) = pathbuf.file_name() {
                 self.change_directory(&parent.to_string_lossy(), false)?
@@ -1064,12 +1187,24 @@ impl App {
         Ok(self)
     }
 
+    fn call0(mut self, command: Command) -> Result<Self> {
+        self.logs_hidden = true;
+        self.msg_out.push_back(MsgOut::Call0(command));
+        Ok(self)
+    }
+
     fn call_silently(mut self, command: Command) -> Result<Self> {
         self.logs_hidden = true;
         self.msg_out.push_back(MsgOut::CallSilently(command));
         Ok(self)
     }
 
+    fn call_silently0(mut self, command: Command) -> Result<Self> {
+        self.logs_hidden = true;
+        self.msg_out.push_back(MsgOut::CallSilently0(command));
+        Ok(self)
+    }
+
     fn bash_exec(self, script: String) -> Result<Self> {
         self.call(Command {
             command: "bash".into(),
@@ -1077,6 +1212,13 @@ impl App {
         })
     }
 
+    fn bash_exec0(self, script: String) -> Result<Self> {
+        self.call0(Command {
+            command: "bash".into(),
+            args: vec!["-c".into(), script],
+        })
+    }
+
     fn bash_exec_silently(self, script: String) -> Result<Self> {
         self.call_silently(Command {
             command: "bash".into(),
@@ -1084,6 +1226,13 @@ impl App {
         })
     }
 
+    fn bash_exec_silently0(self, script: String) -> Result<Self> {
+        self.call_silently0(Command {
+            command: "bash".into(),
+            args: vec!["-c".into(), script],
+        })
+    }
+
     fn call_lua(mut self, func: String) -> Result<Self> {
         self.logs_hidden = true;
         self.msg_out.push_back(MsgOut::CallLua(func));
@@ -1122,9 +1271,11 @@ impl App {
             dir.parent.clone(),
             dir.focused_node().map(|n| n.relative_path.clone()),
         )?;
+
         if dir.parent == self.pwd {
             self.directory_buffer = Some(dir);
-        }
+        };
+
         Ok(self)
     }
 
@@ -1145,10 +1296,7 @@ impl App {
     }
 
     pub fn select_path(mut self, path: String) -> Result<Self> {
-        let mut path = PathBuf::from(path);
-        if path.is_relative() {
-            path = PathBuf::from(self.pwd.clone()).join(path);
-        }
+        let path = PathBuf::from(path).absolutize()?.to_path_buf();
         let parent = path.parent().map(|p| p.to_string_lossy().to_string());
         let filename = path.file_name().map(|p| p.to_string_lossy().to_string());
         if let (Some(p), Some(n)) = (parent, filename) {
@@ -1175,10 +1323,7 @@ impl App {
     }
 
     pub fn un_select_path(mut self, path: String) -> Result<Self> {
-        let mut pathbuf = PathBuf::from(path);
-        if pathbuf.is_relative() {
-            pathbuf = PathBuf::from(self.pwd.clone()).join(pathbuf);
-        }
+        let pathbuf = PathBuf::from(path).absolutize()?.to_path_buf();
         self.selection
             .retain(|n| PathBuf::from(&n.absolute_path) != pathbuf);
         Ok(self)
@@ -1215,10 +1360,7 @@ impl App {
     }
 
     fn toggle_selection_by_path(self, path: String) -> Result<Self> {
-        let mut pathbuf = PathBuf::from(&path);
-        if pathbuf.is_relative() {
-            pathbuf = PathBuf::from(self.pwd.clone()).join(pathbuf);
-        }
+        let pathbuf = PathBuf::from(&path).absolutize()?.to_path_buf();
         if self
             .selection
             .iter()
@@ -1350,6 +1492,58 @@ impl App {
         Ok(self)
     }
 
+    pub fn search_fuzzy(mut self, pattern: String) -> Result<Self> {
+        let rf = self
+            .explorer_config
+            .searcher
+            .as_ref()
+            .map(|s| s.recoverable_focus.clone())
+            .unwrap_or_else(|| self.focused_node().map(|n| n.absolute_path.clone()));
+
+        self.explorer_config.searcher = Some(NodeSearcher::new(pattern, rf));
+        Ok(self)
+    }
+
+    fn search_fuzzy_from_input(self) -> Result<Self> {
+        if let Some(pattern) = self.input.buffer.as_ref().map(Input::to_string) {
+            self.search_fuzzy(pattern)
+        } else {
+            Ok(self)
+        }
+    }
+
+    fn accept_search(mut self) -> Result<Self> {
+        let focus = self
+            .directory_buffer
+            .as_ref()
+            .and_then(|dir| dir.focused_node())
+            .map(|n| n.relative_path.clone());
+
+        self.explorer_config.searcher = None;
+        self = self.explore_pwd()?;
+
+        if let Some(f) = focus {
+            self = self.focus_path(&f, true)?;
+        }
+        Ok(self)
+    }
+
+    fn cancel_search(mut self) -> Result<Self> {
+        let focus = self
+            .explorer_config
+            .searcher
+            .as_ref()
+            .and_then(|s| s.recoverable_focus.clone());
+
+        self.explorer_config.searcher = None;
+        self = self.explore_pwd()?;
+
+        if let Some(f) = focus {
+            self = self.focus_path(&f, true)?;
+        }
+        Ok(self)
+    }
+
     fn enable_mouse(mut self) -> Result<Self> {
         self.msg_out.push_back(MsgOut::EnableMouse);
         Ok(self)
@@ -1462,48 +1656,48 @@ impl App {
         }
     }
 
-    pub fn directory_nodes_str(&self) -> String {
+    pub fn directory_nodes_str(&self, delimiter: char) -> String {
         self.directory_buffer
             .as_ref()
             .map(|d| {
                 d.nodes
                     .iter()
-                    .map(|n| format!("{}\n", n.absolute_path))
+                    .map(|n| format!("{}{}", n.absolute_path, delimiter))
                     .collect::<Vec<String>>()
                     .join("")
             })
             .unwrap_or_default()
     }
 
-    pub fn pwd_str(&self) -> String {
-        format!("{}\n", &self.pwd)
+    pub fn pwd_str(&self, delimiter: char) -> String {
+        format!("{}{}", &self.pwd, delimiter)
     }
 
-    pub fn selection_str(&self) -> String {
+    pub fn selection_str(&self, delimiter: char) -> String {
         self.selection
             .iter()
-            .map(|n| format!("{}\n", n.absolute_path))
+            .map(|n| format!("{}{}", n.absolute_path, delimiter))
             .collect::<Vec<String>>()
             .join("")
     }
 
-    pub fn result_str(&self) -> String {
+    pub fn result_str(&self, delimiter: char) -> String {
         self.result()
             .into_iter()
-            .map(|n| format!("{}\n", n.absolute_path))
+            .map(|n| format!("{}{}", n.absolute_path, delimiter))
             .collect::<Vec<String>>()
             .join("")
     }
 
-    pub fn logs_str(&self) -> String {
+    pub fn logs_str(&self, delimiter: char) -> String {
         self.logs
             .iter()
-            .map(|l| format!("{}\n", l))
+            .map(|l| format!("{}{}", l, delimiter))
             .collect::<Vec<String>>()
             .join("")
     }
 
-    pub fn global_help_menu_str(&self) -> String {
+    pub fn global_help_menu_str(&self, delimiter: char) -> String {
         let builtin = self.config.modes.builtin.clone();
         let custom = self.config.modes.custom.clone();
         let read_only = self.config.general.read_only;
@@ -1519,53 +1713,53 @@ impl App {
                 .help_menu()
                 .iter()
                 .map(|l| match l {
-                    HelpMenuLine::Paragraph(p) => format!("\t{}\n", p),
+                    HelpMenuLine::Paragraph(p) => format!("\t{}{}", p, delimiter),
                     HelpMenuLine::KeyMap(k, remaps, h) => {
                         let remaps = remaps.join(", ");
-                        format!(" {:15} | {:25} | {}\n", k, remaps, h)
+                        format!(" {:15} | {:25} | {}{}", k, remaps, h , delimiter)
                     }
                 })
                 .collect::<Vec<String>>()
                 .join("");
 
             format!(
-                "### {}\n\n key             | remaps                    | action\n --------------- | ------------------------- | ------\n{}\n",
-                name, help
+                "### {}{d}{d} key             | remaps                    | action\n --------------- | ------------------------- | ------{d}{}{d}",
+                name, help, d = delimiter
             )
         })
         .collect::<Vec<String>>()
-        .join("\n")
+        .join(&delimiter.to_string())
     }
 
-    pub fn history_str(&self) -> String {
+    pub fn history_str(&self, delimiter: char) -> String {
         self.history
             .paths
             .iter()
-            .map(|p| format!("{}\n", &p))
+            .map(|p| format!("{}{}", &p, delimiter))
             .collect::<Vec<String>>()
             .join("")
     }
 
-    pub fn write_pipes(&self) -> Result<()> {
+    pub fn write_pipes(&self, delimiter: char) -> Result<()> {
         fs::create_dir_all(self.pipe.path.clone())?;
-        fs::write(&self.pipe.msg_in, "")?;
+        fs::write(&self.pipe.msg_in, [delimiter as u8])?;
 
-        let selection_str = self.selection_str();
+        let selection_str = self.selection_str(delimiter);
         fs::write(&self.pipe.selection_out, selection_str)?;
 
-        let history_str = self.history_str();
+        let history_str = self.history_str(delimiter);
         fs::write(&self.pipe.history_out, history_str)?;
 
-        let directory_nodes_str = self.directory_nodes_str();
+        let directory_nodes_str = self.directory_nodes_str(delimiter);
         fs::write(&self.pipe.directory_nodes_out, directory_nodes_str)?;
 
-        let logs_str = self.logs_str();
+        let logs_str = self.logs_str(delimiter);
         fs::write(&self.pipe.logs_out, logs_str)?;
 
-        let result_str = self.result_str();
+        let result_str = self.result_str(delimiter);
         fs::write(&self.pipe.result_out, result_str)?;
 
-        let global_help_menu_str = self.global_help_menu_str();
+        let global_help_menu_str = self.global_help_menu_str(delimiter);
         fs::write(&self.pipe.global_help_menu_out, global_help_menu_str)?;
 
         Ok(())
@@ -1592,6 +1786,8 @@ impl App {
         LuaContextHeavy {
             version: self.version.clone(),
             pwd: self.pwd.clone(),
+            initial_pwd: self.initial_pwd.clone(),
+            vroot: self.vroot.clone(),
             focused_node: self.focused_node().cloned(),
             directory_buffer: self.directory_buffer.clone(),
             selection: self.selection.clone(),
@@ -1610,6 +1806,8 @@ impl App {
         LuaContextLight {
             version: self.version.clone(),
             pwd: self.pwd.clone(),
+            initial_pwd: self.initial_pwd.clone(),
+            vroot: self.vroot.clone(),
             focused_node: self.focused_node().cloned(),
             selection: self.selection.clone(),
             mode: self.mode.clone(),
