@@ -1,8 +1,7 @@
 #![allow(clippy::too_many_arguments)]
 
 use std::env;
-
-use xplr::cli::Cli;
+use xplr::cli::{self, Cli};
 use xplr::runner;
 
 fn main() {
@@ -20,7 +19,11 @@ fn main() {
     --                           Denotes the end of command-line flags and options
         --force-focus            Focuses on the given <PATH>, even if it is a directory
     -h, --help                   Prints help information
-        --print-pwd-as-result    Prints the present working directory when quitting 
+    -m, --pipe-msg-in            Helps safely passing messages to the active xplr
+                                   session, use %%, %s and %q as the placeholders
+    -M, --print-msg-in           Like --pipe-msg-in, but prints the message instead of
+                                   passing to the active xplr session
+        --print-pwd-as-result    Prints the present working directory when quitting
                                    with `PrintResultAndQuit`
         --read-only              Enables read-only mode (config.general.read_only)
         --read0                  Reads paths separated using the null character (\0)
@@ -32,7 +35,8 @@ fn main() {
     -c, --config <PATH>             Specifies a custom config file (default is
                                       "$HOME/.config/xplr/init.lua")
     -C, --extra-config <PATH>...    Specifies extra config files to load
-        --on-load <MESSAGE>...      Sends messages when xplr loads"###;
+        --on-load <MESSAGE>...      Sends messages when xplr loads
+        --vroot <PATH>              Treats the specified path as the virtual root"###;
 
         let args = r###"
     <PATH>            Path to focus on, or enter if directory, (default is `.`)
@@ -53,17 +57,19 @@ fn main() {
         println!("{}", help);
     } else if cli.version {
         println!("xplr {}", xplr::app::VERSION);
+    } else if !cli.pipe_msg_in.is_empty() {
+        if let Err(err) = cli::pipe_msg_in(cli.pipe_msg_in) {
+            eprintln!("error: {}", err);
+            std::process::exit(1);
+        }
+    } else if !cli.print_msg_in.is_empty() {
+        if let Err(err) = cli::print_msg_in(cli.print_msg_in) {
+            eprintln!("error: {}", err);
+            std::process::exit(1);
+        }
     } else {
-        let write0 = cli.write0;
         match runner::from_cli(cli).and_then(|a| a.run()) {
-            Ok(Some(mut out)) => {
-                if write0 {
-                    out = out
-                        .trim_end()
-                        .chars()
-                        .map(|c| if c == '\n' { '\0' } else { c })
-                        .collect();
-                }
+            Ok(Some(out)) => {
                 print!("{}", out);
             }
             Ok(None) => {}
