@@ -8,10 +8,11 @@ pub enum MsgOut {
     ExploreParentsAsync,
     Refresh,
     ClearScreen,
-    Quit,
     Debug(String),
     Call(Command),
+    Call0(Command),
     CallSilently(Command),
+    CallSilently0(Command),
     CallLua(String),
     CallLuaSilently(String),
     LuaEval(String),
@@ -23,6 +24,11 @@ pub enum MsgOut {
     StartFifo(String),
     StopFifo,
     ToggleFifo(String),
+    ScrollUp,
+    ScrollDown,
+    ScrollUpHalf,
+    ScrollDownHalf,
+    Quit,
     PrintPwdAndQuit,
     PrintFocusPathAndQuit,
     PrintSelectionAndQuit,
