@@ -9,6 +9,7 @@ use crate::lua;
 use crate::pipe;
 use crate::pwd_watcher;
 use crate::ui;
+use crate::yaml;
 use anyhow::{bail, Error, Result};
 use crossterm::event;
 use crossterm::execute;
@@ -16,16 +17,18 @@ use crossterm::terminal as term;
 use mlua::LuaSerdeExt;
 use mlua::Value;
 use std::fs;
+use std::fs::File;
 use std::io::Write;
 use std::path::PathBuf;
-use std::process::{Command, ExitStatus, Stdio};
+use std::process::{Command, Stdio};
 use std::sync::mpsc;
 use tui::backend::CrosstermBackend;
 use tui::Terminal;
+use tui_input::Input;
 
 pub fn get_tty() -> Result<fs::File> {
     let tty = "/dev/tty";
-    match fs::OpenOptions::new().read(true).write(true).open(&tty) {
+    match fs::OpenOptions::new().read(true).write(true).open(tty) {
         Ok(f) => Ok(f),
         Err(e) => {
             bail!(format!("Failed to open {}. {}", tty, e))
@@ -60,7 +63,30 @@ fn call_lua_heavy(
     lua::call(lua, func, arg)
 }
 
-fn call(app: &app::App, cmd: app::Command, silent: bool) -> Result<ExitStatus> {
+fn call(
+    mut app: app::App,
+    cmd: app::Command,
+    silent: bool,
+    terminal: &mut Terminal<CrosstermBackend<File>>,
+    event_reader: &mut EventReader,
+    mouse_enabled: &mut bool,
+    delimiter: char,
+) -> Result<app::App> {
+    if !silent {
+        if *mouse_enabled {
+            execute!(terminal.backend_mut(), event::DisableMouseCapture)
+                .unwrap_or_default();
+        }
+
+        event_reader.stop();
+
+        terminal.clear()?;
+        terminal.set_cursor(0, 0)?;
+        term::disable_raw_mode()?;
+        terminal.show_cursor()?;
+    }
+
+    app.write_pipes(delimiter)?;
     let focus_index = app
         .directory_buffer
         .as_ref()
@@ -74,17 +100,20 @@ fn call(app: &app::App, cmd: app::Command, silent: bool) -> Result<ExitStatus> {
         (get_tty()?.into(), get_tty()?.into(), get_tty()?.into())
     };
 
-    Command::new(cmd.command.clone())
-        .env("XPLR_APP_VERSION", app.version.clone())
+    let input_buffer = app
+        .input
+        .buffer
+        .as_ref()
+        .map(Input::to_string)
+        .unwrap_or_default();
+
+    let status = Command::new(cmd.command.clone())
+        .env("XPLR", &app.bin)
+        .env("XPLR_VROOT", &app.vroot.clone().unwrap_or_default())
+        .env("XPLR_APP_VERSION", &app.version)
         .env("XPLR_PID", &app.pid.to_string())
-        .env(
-            "XPLR_INPUT_BUFFER",
-            app.input
-                .buffer
-                .as_ref()
-                .map(|i| i.value().to_string())
-                .unwrap_or_default(),
-        )
+        .env("XPLR_INPUT_BUFFER", input_buffer)
+        .env("XPLR_INITIAL_PWD", &app.initial_pwd)
         .env("XPLR_FOCUS_PATH", app.focused_node_str())
         .env("XPLR_FOCUS_INDEX", focus_index)
         .env("XPLR_SESSION_PATH", &app.session_path)
@@ -107,7 +136,49 @@ fn call(app: &app::App, cmd: app::Command, silent: bool) -> Result<ExitStatus> {
         .stderr(stderr)
         .args(cmd.args)
         .status()
-        .map_err(Error::new)
+        .map(|s| {
+            if s.success() {
+                Ok(())
+            } else {
+                Err(format!("process exited with code {}", &s))
+            }
+        })
+        .unwrap_or_else(|e| Err(e.to_string()));
+
+    match pipe::read_all(&app.pipe.msg_in, delimiter) {
+        Ok(msgs) => {
+            app = app.handle_batch_external_msgs(msgs)?;
+        }
+        Err(err) => {
+            app = app.log_error(err.to_string())?;
+        }
+    };
+
+    app.cleanup_pipes()?;
+
+    if let Err(e) = status {
+        app = app.log_error(e)?;
+    };
+
+    if !silent {
+        terminal.clear()?;
+        term::enable_raw_mode()?;
+        terminal.hide_cursor()?;
+        event_reader.start();
+
+        if *mouse_enabled {
+            match execute!(terminal.backend_mut(), event::EnableMouseCapture) {
+                Ok(_) => {
+                    *mouse_enabled = true;
+                }
+                Err(e) => {
+                    app = app.log_error(e.to_string())?;
+                }
+            }
+        }
+    }
+
+    Ok(app)
 }
 
 fn start_fifo(path: &str, focus_path: &str) -> Result<fs::File> {
@@ -121,6 +192,8 @@ fn start_fifo(path: &str, focus_path: &str) -> Result<fs::File> {
 }
 
 pub struct Runner {
+    bin: String,
+    vroot: Option<PathBuf>,
     pwd: PathBuf,
     focused_path: Option<PathBuf>,
     config_file: Option<PathBuf>,
@@ -129,6 +202,7 @@ pub struct Runner {
     read_only: bool,
     print_pwd_as_result: bool,
     selection: Vec<PathBuf>,
+    delimiter: char,
 }
 
 impl Runner {
@@ -139,24 +213,22 @@ impl Runner {
 
     /// Create a new runner object passing the given arguments
     pub fn from_cli(cli: Cli) -> Result<Self> {
-        let basedir = get_current_dir()?;
-        let basedir_clone = basedir.clone();
-        let mut paths = cli.paths.into_iter().map(|p| {
-            if p.is_relative() {
-                basedir_clone.join(p)
-            } else {
-                p
-            }
-        });
-        let mut pwd = paths.next().unwrap_or_else(|| basedir.clone());
+        let currdir = get_current_dir()?;
+        let mut paths = cli.paths.into_iter();
+        let mut pwd = paths
+            .next()
+            .or_else(|| cli.vroot.clone())
+            .unwrap_or_else(|| currdir.clone());
         let mut focused_path = None;
 
         if cli.force_focus || pwd.is_file() {
             focused_path = pwd.file_name().map(|p| p.into());
-            pwd = pwd.parent().map(|p| p.into()).unwrap_or(basedir);
+            pwd = pwd.parent().map(|p| p.into()).unwrap_or(currdir);
         }
 
         Ok(Self {
+            bin: cli.bin,
+            vroot: cli.vroot,
             pwd,
             focused_path,
             config_file: cli.config,
@@ -165,6 +237,7 @@ impl Runner {
             read_only: cli.read_only,
             print_pwd_as_result: cli.print_pwd_as_result,
             selection: paths.collect(),
+            delimiter: if cli.write0 { '\0' } else { '\n' },
         })
     }
 
@@ -172,8 +245,14 @@ impl Runner {
     pub fn run(self) -> Result<Option<String>> {
         // Why unsafe? See https://github.com/sayanarijit/xplr/issues/309
         let lua = unsafe { mlua::Lua::unsafe_new() };
-        let mut app =
-            app::App::create(self.pwd, &lua, self.config_file, self.extra_config_files)?;
+        let mut app = app::App::create(
+            self.bin,
+            self.vroot,
+            self.pwd,
+            &lua,
+            self.config_file,
+            self.extra_config_files,
+        )?;
         app.config.general.read_only = self.read_only;
 
         fs::create_dir_all(app.session_path.clone())?;
@@ -244,6 +323,7 @@ impl Runner {
         let backend = CrosstermBackend::new(stdout);
         let mut terminal = Terminal::new(backend)?;
         terminal.hide_cursor()?;
+        terminal.clear()?;
 
         // Threads
         pwd_watcher::keep_watching(app.pwd.as_ref(), tx_msg_in.clone(), rx_pwd_watcher)?;
@@ -251,10 +331,16 @@ impl Runner {
         event_reader.start();
 
         // Enqueue on_load messages
-        for msg in self.on_load {
-            tx_msg_in.send(app::Task::new(app::MsgIn::External(msg), None))?;
+        for msg in app.hooks.on_load.iter().chain(self.on_load.iter()) {
+            tx_msg_in.send(app::Task::new(app::MsgIn::External(msg.clone()), None))?;
         }
 
+        // Refresh screen once after loading
+        tx_msg_in.send(app::Task::new(
+            app::MsgIn::External(app::ExternalMsg::Refresh),
+            None,
+        ))?;
+
         'outer: for task in rx_msg_in {
             match app.handle_task(task) {
                 Ok(a) => {
@@ -264,8 +350,6 @@ impl Runner {
                         match msg {
                             // NOTE: Do not schedule critical tasks via tx_msg_in in this loop.
                             // Try handling them immediately.
-                            //
-                            // TODO: Remove boilerplate code.
                             Enque(task) => {
                                 tx_msg_in.send(task)?;
                             }
@@ -276,46 +360,70 @@ impl Runner {
                             }
 
                             PrintPwdAndQuit => {
-                                result = Ok(Some(format!("{}\n", app.pwd)));
+                                result = Ok(Some(app.pwd_str(self.delimiter)));
                                 break 'outer;
                             }
 
                             PrintFocusPathAndQuit => {
-                                result = Ok(app
-                                    .focused_node()
-                                    .map(|n| format!("{}\n", n.absolute_path)));
+                                result = Ok(app.focused_node().map(|n| {
+                                    format!("{}{}", n.absolute_path, self.delimiter)
+                                }));
                                 break 'outer;
                             }
 
                             PrintSelectionAndQuit => {
-                                result = Ok(Some(app.selection_str()));
+                                result = Ok(Some(app.selection_str(self.delimiter)));
                                 break 'outer;
                             }
 
                             PrintResultAndQuit => {
                                 result = if self.print_pwd_as_result {
-                                    Ok(Some(app.pwd_str()))
+                                    Ok(Some(app.pwd_str(self.delimiter)))
                                 } else {
-                                    Ok(Some(app.result_str()))
+                                    Ok(Some(app.result_str(self.delimiter)))
                                 };
 
                                 break 'outer;
                             }
 
                             PrintAppStateAndQuit => {
-                                let out = serde_yaml::to_string(&app)?;
+                                let out = yaml::to_string(&app)?;
                                 result = Ok(Some(out));
                                 break 'outer;
                             }
 
                             Debug(path) => {
-                                fs::write(&path, serde_yaml::to_string(&app)?)?;
+                                fs::write(&path, yaml::to_string(&app)?)?;
                             }
 
                             ClearScreen => {
                                 terminal.clear()?;
                             }
 
+                            ScrollUp => {
+                                app = app.focus_previous_by_relative_index(
+                                    terminal.size()?.height as usize,
+                                )?;
+                            }
+
+                            ScrollDown => {
+                                app = app.focus_next_by_relative_index(
+                                    terminal.size()?.height as usize,
+                                )?;
+                            }
+
+                            ScrollUpHalf => {
+                                app = app.focus_next_by_relative_index(
+                                    terminal.size()?.height as usize / 2,
+                                )?;
+                            }
+
+                            ScrollDownHalf => {
+                                app = app.focus_next_by_relative_index(
+                                    terminal.size()?.height as usize / 2,
+                                )?;
+                            }
+
                             ExplorePwdAsync => {
                                 explorer::explore_async(
                                     app.explorer_config.clone(),
@@ -347,16 +455,25 @@ impl Runner {
                             }
 
                             Refresh => {
-                                // Fifo
                                 let focus = app.focused_node();
                                 if focus != last_focus.as_ref() {
+                                    last_focus = focus.cloned();
+
+                                    // Fifo
                                     if let Some(ref mut file) = fifo {
                                         writeln!(file, "{}", app.focused_node_str())?;
                                     };
-                                    last_focus = focus.cloned();
+
+                                    // Hooks
+                                    if !app.hooks.on_focus_change.is_empty() {
+                                        let msgs = app.hooks.on_focus_change.clone();
+                                        app = app.handle_batch_external_msgs(msgs)?
+                                    }
                                 }
 
                                 if app.pwd != last_pwd {
+                                    last_pwd = app.pwd.clone();
+
                                     // $PWD watcher
                                     tx_pwd_watcher.send(app.pwd.clone())?;
 
@@ -370,7 +487,11 @@ impl Runner {
                                         )?;
                                     }
 
-                                    last_pwd = app.pwd.clone();
+                                    // Hooks
+                                    if !app.hooks.on_directory_change.is_empty() {
+                                        let msgs = app.hooks.on_directory_change.clone();
+                                        app = app.handle_batch_external_msgs(msgs)?
+                                    }
                                 }
 
                                 // UI
@@ -467,37 +588,6 @@ impl Runner {
                                 };
                             }
 
-                            CallSilently(cmd) => {
-                                app.write_pipes()?;
-                                let status = call(&app, cmd, true)
-                                    .map(|s| {
-                                        if s.success() {
-                                            Ok(())
-                                        } else {
-                                            Err(format!(
-                                                "process exited with code {}",
-                                                &s
-                                            ))
-                                        }
-                                    })
-                                    .unwrap_or_else(|e| Err(e.to_string()));
-
-                                match pipe::read_all(&app.pipe.msg_in) {
-                                    Ok(msgs) => {
-                                        app = app.handle_batch_external_msgs(msgs)?;
-                                    }
-                                    Err(err) => {
-                                        app = app.log_error(err.to_string())?;
-                                    }
-                                };
-
-                                app.cleanup_pipes()?;
-
-                                if let Err(e) = status {
-                                    app = app.log_error(e.to_string())?;
-                                };
-                            }
-
                             CallLua(func) => {
                                 execute!(
                                     terminal.backend_mut(),
@@ -661,67 +751,51 @@ impl Runner {
                             }
 
                             Call(cmd) => {
-                                execute!(
-                                    terminal.backend_mut(),
-                                    event::DisableMouseCapture
-                                )
-                                .unwrap_or_default();
+                                app = call(
+                                    app,
+                                    cmd,
+                                    false,
+                                    &mut terminal,
+                                    &mut event_reader,
+                                    &mut mouse_enabled,
+                                    '\n',
+                                )?;
+                            }
 
-                                event_reader.stop();
+                            Call0(cmd) => {
+                                app = call(
+                                    app,
+                                    cmd,
+                                    false,
+                                    &mut terminal,
+                                    &mut event_reader,
+                                    &mut mouse_enabled,
+                                    '\0',
+                                )?;
+                            }
 
-                                terminal.clear()?;
-                                terminal.set_cursor(0, 0)?;
-                                term::disable_raw_mode()?;
-                                terminal.show_cursor()?;
+                            CallSilently(cmd) => {
+                                app = call(
+                                    app,
+                                    cmd,
+                                    true,
+                                    &mut terminal,
+                                    &mut event_reader,
+                                    &mut mouse_enabled,
+                                    '\n',
+                                )?;
+                            }
 
-                                app.write_pipes()?;
-                                let status = call(&app, cmd, false)
-                                    .map(|s| {
-                                        if s.success() {
-                                            Ok(())
-                                        } else {
-                                            Err(format!(
-                                                "process exited with code {}",
-                                                &s
-                                            ))
-                                        }
-                                    })
-                                    .unwrap_or_else(|e| Err(e.to_string()));
-
-                                // TODO remove duplicate segment
-                                match pipe::read_all(&app.pipe.msg_in) {
-                                    Ok(msgs) => {
-                                        app = app.handle_batch_external_msgs(msgs)?;
-                                    }
-                                    Err(err) => {
-                                        app = app.log_error(err.to_string())?;
-                                    }
-                                };
-
-                                app.cleanup_pipes()?;
-
-                                if let Err(e) = status {
-                                    app = app.log_error(e.to_string())?;
-                                };
-
-                                terminal.clear()?;
-                                term::enable_raw_mode()?;
-                                terminal.hide_cursor()?;
-                                event_reader.start();
-
-                                if mouse_enabled {
-                                    match execute!(
-                                        terminal.backend_mut(),
-                                        event::EnableMouseCapture
-                                    ) {
-                                        Ok(_) => {
-                                            mouse_enabled = true;
-                                        }
-                                        Err(e) => {
-                                            app = app.log_error(e.to_string())?;
-                                        }
-                                    }
-                                }
+                            CallSilently0(cmd) => {
+                                app = call(
+                                    app,
+                                    cmd,
+                                    true,
+                                    &mut terminal,
+                                    &mut event_reader,
+                                    &mut mouse_enabled,
+                                    '\0',
+                                )?;
                             }
                         };
                     }
