@@ -104,7 +104,7 @@ pub enum ContentBody {
     },
 }
 
-#[derive(Debug, Clone, Serialize, Deserialize, PartialEq)]
+#[derive(Debug, Clone, Serialize, Deserialize, PartialEq, Eq)]
 #[serde(deny_unknown_fields)]
 pub enum Layout {
     Nothing,
@@ -251,7 +251,7 @@ impl Modifier {
     }
 }
 
-#[derive(Debug, PartialEq, Clone, Default, Serialize, Deserialize)]
+#[derive(Debug, PartialEq, Eq, Clone, Default, Serialize, Deserialize)]
 #[serde(deny_unknown_fields)]
 pub struct Style {
     pub fg: Option<Color>,
@@ -669,6 +669,24 @@ fn draw_table<B: Backend>(
         .map(|c| c.to_tui(screen_size, layout_size))
         .collect();
 
+    let pwd = if let Some(vroot) = app.vroot.as_ref() {
+        app.pwd.strip_prefix(vroot).unwrap_or(&app.pwd)
+    } else {
+        &app.pwd
+    }
+    .trim_matches('/')
+    .replace('\\', "\\\\")
+    .replace('\n', "\\n");
+
+    let vroot_indicator = if app.vroot.is_some() { "vroot:" } else { "" };
+
+    let node_count = app.directory_buffer.as_ref().map(|d| d.total).unwrap_or(0);
+    let node_count = if node_count == 0 {
+        String::new()
+    } else {
+        format!("({node_count}) ")
+    };
+
     let table = Table::new(rows)
         .widths(&table_constraints)
         .style(app_config.general.table.style.to_owned().into())
@@ -676,14 +694,7 @@ fn draw_table<B: Backend>(
         .column_spacing(app_config.general.table.col_spacing.unwrap_or_default())
         .block(block(
             config,
-            format!(
-                " {} ({}) ",
-                app.pwd,
-                app.directory_buffer
-                    .as_ref()
-                    .map(|d| d.total)
-                    .unwrap_or_default()
-            ),
+            format!(" {vroot_indicator}/{pwd} {node_count}"),
         ));
 
     let table = table.to_owned().header(
@@ -727,13 +738,19 @@ fn draw_selection<B: Backend>(
         .rev()
         .take((layout_size.height.max(2) - 2).into())
         .rev()
-        .map(|n| n.absolute_path.to_owned())
+        .map(|n| n.absolute_path.replace('\\', "\\\\").replace('\n', "\\n"))
         .map(ListItem::new)
         .collect();
 
     // Selected items
+    let selection_count = if selection_count == 0 {
+        String::new()
+    } else {
+        format!("({selection_count}) ")
+    };
+
     let selection_list = List::new(selection)
-        .block(block(config, format!(" Selection ({}) ", selection_count)));
+        .block(block(config, format!(" Selection {selection_count}")));
 
     f.render_widget(selection_list, layout_size);
 }
@@ -746,10 +763,12 @@ fn draw_help_menu<B: Backend>(
     _: &Lua,
 ) {
     let panel_config = &app.config.general.panel_ui;
+
     let config = panel_config
         .default
         .to_owned()
         .extend(&panel_config.help_menu);
+
     let help_menu_rows = app
         .mode
         .help_menu()
@@ -797,31 +816,23 @@ fn draw_input_buffer<B: Backend>(
             .to_owned()
             .extend(&panel_config.input_and_logs);
 
-        let cursor_offset_left = if config
+        let cursor_offset_left = config
             .borders
             .as_ref()
             .map(|b| b.contains(&Border::Left))
-            .unwrap_or(false)
-        {
-            1
-        } else {
-            0
-        } + app.input.prompt.chars().count() as u16;
+            .unwrap_or(false) as u16
+            + app.input.prompt.chars().count() as u16;
 
-        let cursor_offset_right = if config
+        let cursor_offset_right = config
             .borders
             .as_ref()
             .map(|b| b.contains(&Border::Right))
-            .unwrap_or(false)
-        {
-            2
-        } else {
-            1
-        };
+            .unwrap_or(false) as u16
+            + 1;
 
         let offset_width = cursor_offset_left + cursor_offset_right;
         let width = layout_size.width.max(offset_width) - offset_width;
-        let scroll = (input.cursor() as u16).max(width) - width;
+        let scroll = input.visual_scroll(width.into()) as u16;
 
         let input_buf = Paragraph::new(Spans::from(vec![
             Span::styled(
@@ -839,7 +850,9 @@ fn draw_input_buffer<B: Backend>(
         f.render_widget(input_buf, layout_size);
         f.set_cursor(
             // Put cursor past the end of the input text
-            layout_size.x + (input.cursor() as u16).min(width) + cursor_offset_left,
+            layout_size.x
+                + (input.visual_cursor() as u16).min(width)
+                + cursor_offset_left,
             // Move one line down, from the border to the input line
             layout_size.y + 1,
         );
@@ -861,6 +874,12 @@ fn draw_sort_n_filter<B: Backend>(
     let ui = app.config.general.sort_and_filter_ui.to_owned();
     let filter_by: &IndexSet<NodeFilterApplicable> = &app.explorer_config.filters;
     let sort_by: &IndexSet<NodeSorterApplicable> = &app.explorer_config.sorters;
+    let search = app
+        .explorer_config
+        .searcher
+        .as_ref()
+        .map(|s| s.pattern.clone());
+
     let defaultui = &ui.default_identifier;
     let forwardui = defaultui
         .to_owned()
@@ -886,25 +905,45 @@ fn draw_sort_n_filter<B: Backend>(
                 })
                 .unwrap_or((Span::raw("f"), Span::raw("")))
         })
-        .chain(sort_by.iter().map(|s| {
-            let direction = if s.reverse { &reverseui } else { &forwardui };
+        .chain(
+            sort_by
+                .iter()
+                .map(|s| {
+                    let direction = if s.reverse { &reverseui } else { &forwardui };
 
-            ui.sorter_identifiers
-                .get(&s.sorter)
+                    ui.sorter_identifiers
+                        .get(&s.sorter)
+                        .map(|u| {
+                            let ui = defaultui.to_owned().extend(u);
+                            (
+                                Span::styled(
+                                    ui.format.to_owned().unwrap_or_default(),
+                                    ui.style.into(),
+                                ),
+                                Span::styled(
+                                    direction.format.to_owned().unwrap_or_default(),
+                                    direction.style.to_owned().into(),
+                                ),
+                            )
+                        })
+                        .unwrap_or((Span::raw("s"), Span::raw("")))
+                })
+                .take(if search.is_some() { 0 } else { sort_by.len() }),
+        )
+        .chain(search.iter().map(|s| {
+            ui.search_identifier
+                .as_ref()
                 .map(|u| {
                     let ui = defaultui.to_owned().extend(u);
                     (
                         Span::styled(
                             ui.format.to_owned().unwrap_or_default(),
-                            ui.style.into(),
-                        ),
-                        Span::styled(
-                            direction.format.to_owned().unwrap_or_default(),
-                            direction.style.to_owned().into(),
+                            ui.style.to_owned().into(),
                         ),
+                        Span::styled(s, ui.style.into()),
                     )
                 })
-                .unwrap_or((Span::raw("s"), Span::raw("")))
+                .unwrap_or((Span::raw("/"), Span::raw(s)))
         }))
         .zip(std::iter::repeat(Span::styled(
             ui.separator.format.to_owned().unwrap_or_default(),
@@ -912,12 +951,18 @@ fn draw_sort_n_filter<B: Backend>(
         )))
         .flat_map(|((a, b), c)| vec![a, b, c])
         .collect::<Vec<Span>>();
+
     spans.pop();
 
-    let p = Paragraph::new(Spans::from(spans)).block(block(
-        config,
-        format!(" Sort & filter ({}) ", filter_by.len() + sort_by.len()),
-    ));
+    let item_count = filter_by.len() + sort_by.len();
+    let item_count = if item_count == 0 {
+        String::new()
+    } else {
+        format!("({item_count}) ")
+    };
+
+    let p = Paragraph::new(Spans::from(spans))
+        .block(block(config, format!(" Sort & filter {item_count}")));
 
     f.render_widget(p, layout_size);
 }
@@ -942,50 +987,52 @@ fn draw_logs<B: Backend>(
             .iter()
             .rev()
             .take(layout_size.height as usize)
-            .map(|l| {
-                let time = l.created_at.format("%r");
-                match l.level {
-                    app::LogLevel::Info => ListItem::new(format!(
-                        "{} | {} | {}",
-                        &time,
-                        &logs_config.info.format.to_owned().unwrap_or_default(),
-                        l.message
-                    ))
-                    .style(logs_config.info.style.to_owned().into()),
+            .map(|log| {
+                let time = log.created_at.format("%r");
+                let cfg = match log.level {
+                    app::LogLevel::Info => &logs_config.info,
+                    app::LogLevel::Warning => &logs_config.warning,
+                    app::LogLevel::Success => &logs_config.success,
+                    app::LogLevel::Error => &logs_config.error,
+                };
 
-                    app::LogLevel::Warning => ListItem::new(format!(
-                        "{} | {} | {}",
-                        &time,
-                        &logs_config.warning.format.to_owned().unwrap_or_default(),
-                        l.message
-                    ))
-                    .style(logs_config.warning.style.to_owned().into()),
+                let prefix =
+                    format!("{}|{}", time, cfg.format.to_owned().unwrap_or_default());
 
-                    app::LogLevel::Success => ListItem::new(format!(
-                        "{} | {} | {}",
-                        &time,
-                        &logs_config.success.format.to_owned().unwrap_or_default(),
-                        l.message
-                    ))
-                    .style(logs_config.success.style.to_owned().into()),
+                let padding = " ".repeat(prefix.chars().count());
 
-                    app::LogLevel::Error => ListItem::new(format!(
-                        "{} | {} | {}",
-                        &time,
-                        &logs_config.error.format.to_owned().unwrap_or_default(),
-                        l.message
-                    ))
-                    .style(logs_config.error.style.to_owned().into()),
-                }
+                let txt = log
+                    .message
+                    .lines()
+                    .enumerate()
+                    .map(|(i, line)| {
+                        if i == 0 {
+                            format!("{}: {}", &prefix, line)
+                        } else {
+                            format!("{}  {}", &padding, line)
+                        }
+                    })
+                    .take(layout_size.height as usize)
+                    .collect::<Vec<_>>()
+                    .join("\n");
+
+                ListItem::new(txt).style(cfg.style.to_owned().into())
             })
             .collect::<Vec<ListItem>>()
     };
 
+    let logs_count = app.logs.len();
+    let logs_count = if logs_count == 0 {
+        String::new()
+    } else {
+        format!("({logs_count}) ")
+    };
+
     let logs_list = List::new(logs).block(block(
         config,
         format!(
-            " Logs ({}) [{}{}] ",
-            app.logs.len(),
+            " Logs {}[{}{}] ",
+            logs_count,
             app.mode.name,
             read_only_indicator(app),
         ),
